@@ -11,24 +11,21 @@ import shutil
 import tempfile
 from textwrap import dedent
 
-import cloudpickle
 from flaky import flaky
-import ipykernel
+from ipykernel.serialize import serialize_object
 from pygments.token import Name
 import pytest
-from qtpy import PYQT4, PYQT5, PYQT_VERSION
+from qtpy import PYQT5, PYQT_VERSION
 from qtpy.QtCore import Qt, QTimer
 from qtpy.QtWidgets import QApplication
-import zmq
 
 from spyder.config.gui import get_color_scheme
 from spyder.config.main import CONF
-from spyder.py3compat import PY2, PY3
+from spyder.py3compat import PY2
 from spyder.plugins.ipythonconsole import (IPythonConsole,
                                            KernelConnectionDialog)
 from spyder.utils.environ import listdict2envdict
 from spyder.utils.ipython.style import create_style_class
-from spyder.utils.programs import TEMPDIR
 from spyder.utils.test import close_message_box
 from spyder.widgets.variableexplorer.collectionseditor import CollectionsEditor
 
@@ -37,8 +34,8 @@ from spyder.widgets.variableexplorer.collectionseditor import CollectionsEditor
 # Constants
 #==============================================================================
 SHELL_TIMEOUT = 20000
+PYQT_WHEEL = PYQT_VERSION > '5.6'
 TEMP_DIRECTORY = tempfile.gettempdir()
-NON_ASCII_DIR = osp.join(TEMP_DIRECTORY, u'測試', u'اختبار')
 
 
 #==============================================================================
@@ -66,44 +63,13 @@ def get_console_background_color(style_sheet):
 # Qt Test Fixtures
 #==============================================================================
 @pytest.fixture
-def ipyconsole(qtbot, request):
-    """IPython console fixture."""
-    # Tests assume inline backend
-    CONF.set('ipython_console', 'pylab/backend', 0)
-
-    # Test the console with a non-ascii temp dir
-    non_ascii_dir = request.node.get_marker('non_ascii_dir')
-    if non_ascii_dir:
-        test_dir = NON_ASCII_DIR
-    else:
-        test_dir = TEMPDIR
-
-    # Instruct the console to not use a stderr file
-    no_stderr_file = request.node.get_marker('no_stderr_file')
-    if no_stderr_file:
-        test_no_stderr = True
-    else:
-        test_no_stderr = False
-
-    # Use the automatic backend if requested
-    auto_backend = request.node.get_marker('auto_backend')
-    if auto_backend:
-        CONF.set('ipython_console', 'pylab/backend', 1)
-
-    # Create the console and a new client
-    console = IPythonConsole(parent=None,
-                             testing=True,
-                             test_dir=test_dir,
-                             test_no_stderr=test_no_stderr)
+def ipyconsole(request):
+    console = IPythonConsole(None, testing=True)
     console.create_new_client()
-
-    # Close callback
     def close_console():
         console.closing_plugin()
         console.close()
     request.addfinalizer(close_console)
-
-    qtbot.addWidget(console)
     console.show()
     return console
 
@@ -111,159 +77,6 @@ def ipyconsole(qtbot, request):
 #==============================================================================
 # Tests
 #==============================================================================
-@pytest.mark.slow
-@flaky(max_runs=3)
-@pytest.mark.auto_backend
-@pytest.mark.skipif(os.name == 'nt' or PYQT4,
-                    reason="It times out sometimes on Windows and it's not needed in PyQt4")
-@pytest.mark.xfail(zmq.__version__ >= '17.0.0' and ipykernel.__version__ <= "4.8.1",
-                   reason="A bug with pyzmq 17 and ipykernel 4.8.1")
-def test_auto_backend(ipyconsole, qtbot):
-    """Test that the automatic backend is working correctly."""
-    # Wait until the window is fully up
-    shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
-
-    # This is here to generate further errors
-    with qtbot.waitSignal(shell.executed):
-        shell.execute("%matplotlib qt5")
-
-    # Assert there are no errors in the console
-    control = ipyconsole.get_focus_widget()
-    assert 'NOTE' not in control.toPlainText()
-    assert 'Error' not in control.toPlainText()
-
-
-@pytest.mark.slow
-@flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
-def test_tab_rename_for_slaves(ipyconsole, qtbot):
-    """Test slave clients are renamed correctly."""
-    # Wait until the window is fully up
-    shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
-
-    cf = ipyconsole.get_current_client().connection_file
-    ipyconsole._create_client_for_kernel(cf, None, None, None)
-    qtbot.wait(1000)
-
-    # Rename slave
-    ipyconsole.rename_tabs_after_change('foo')
-
-    # Assert both clients have the same name
-    assert 'foo' in ipyconsole.get_clients()[0].get_name()
-    assert 'foo' in ipyconsole.get_clients()[1].get_name()
-
-
-@pytest.mark.slow
-@flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
-def test_no_repeated_tabs_name(ipyconsole, qtbot):
-    """Test that tabs can't have repeated given names."""
-    # Rename first client
-    ipyconsole.rename_tabs_after_change('foo')
-
-    # Create a new client and try to rename it
-    ipyconsole.create_new_client()
-    ipyconsole.rename_tabs_after_change('foo')
-
-    # Assert the rename didn't take place
-    client_name = ipyconsole.get_current_client().get_name()
-    assert '2' in client_name
-
-
-@pytest.mark.slow
-@flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
-def test_tabs_preserve_name_after_move(ipyconsole, qtbot):
-    """Test that tabs preserve their names after they are moved."""
-    # Create a new client
-    ipyconsole.create_new_client()
-
-    # Move tabs
-    ipyconsole.tabwidget.tabBar().moveTab(0, 1)
-
-    # Assert the second client is in the first position
-    client_name = ipyconsole.get_clients()[0].get_name()
-    assert '2' in client_name
-
-
-@pytest.mark.slow
-@flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
-def test_conf_env_vars(ipyconsole, qtbot):
-    """Test that kernels have env vars set by our kernel spec."""
-    # Wait until the window is fully up
-    shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
-
-    # Get a CONF env var
-    with qtbot.waitSignal(shell.executed):
-        shell.execute("import os; a = os.environ.get('SPY_SYMPY_O')")
-
-    # Assert we get the assigned value correctly
-    assert shell.get_value('a') == 'False'
-
-
-@pytest.mark.slow
-@flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
-@pytest.mark.no_stderr_file
-def test_no_stderr_file(ipyconsole, qtbot):
-    """Test that consoles can run without an stderr."""
-    # Wait until the window is fully up
-    shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
-
-    # Execute a simple assignment
-    with qtbot.waitSignal(shell.executed):
-        shell.execute('a = 1')
-
-    # Assert we get the assigned value correctly
-    assert shell.get_value('a') == 1
-
-
-@pytest.mark.slow
-@pytest.mark.non_ascii_dir
-@flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
-def test_non_ascii_stderr_file(ipyconsole, qtbot):
-    """Test the creation of a console with a stderr file in a non-ascii dir."""
-    # Wait until the window is fully up
-    shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
-
-    # Execute a simple assignment
-    with qtbot.waitSignal(shell.executed):
-        shell.execute('a = 1')
-
-    # Assert we get the assigned value
-    assert shell.get_value('a') == 1
-
-
-@pytest.mark.slow
-@flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
-def test_console_import_namespace(ipyconsole, qtbot):
-    """Test an import of the form 'from foo import *'."""
-    # Wait until the window is fully up
-    shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
-
-    # Import numpy
-    with qtbot.waitSignal(shell.executed):
-        shell.execute('from numpy import *')
-
-    # Assert we get the e value correctly
-    assert shell.get_value('e') == 2.718281828459045
-
-
-@pytest.mark.slow
 @flaky(max_runs=3)
 def test_console_disambiguation(ipyconsole, qtbot):
     """Test the disambiguation of dedicated consoles."""
@@ -298,7 +111,6 @@ def test_console_disambiguation(ipyconsole, qtbot):
     assert client.get_name() == 'c.py - b/A'
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
 def test_console_coloring(ipyconsole, qtbot):
 
@@ -323,7 +135,6 @@ def test_console_coloring(ipyconsole, qtbot):
     assert console_font_color.strip() == editor_font_color.strip()
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt', reason="It doesn't work on Windows")
 def test_get_env(ipyconsole, qtbot):
@@ -351,7 +162,6 @@ def test_get_env(ipyconsole, qtbot):
     assert env_contents['FOO'] == 'bar'
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt', reason="It doesn't work on Windows")
 def test_get_syspath(ipyconsole, qtbot):
@@ -385,8 +195,7 @@ def test_get_syspath(ipyconsole, qtbot):
         pass
 
 
-@pytest.mark.slow
-@flaky(max_runs=10)
+@flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt', reason="It doesn't work on Windows")
 def test_browse_history_dbg(ipyconsole, qtbot):
     """Test that browsing command history is working while debugging."""
@@ -419,7 +228,6 @@ def test_browse_history_dbg(ipyconsole, qtbot):
     assert '!aa = 10' in control.toPlainText()
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt' or PY2,
                     reason="It times out sometimes on Windows and doesn't work on PY2")
@@ -439,12 +247,11 @@ def test_unicode_vars(ipyconsole, qtbot):
     assert shell.get_value('д') == 10
 
     # Change its value and verify
-    shell.set_value('д', [cloudpickle.dumps(20, protocol=2)])
+    shell.set_value('д', serialize_object(20))
     qtbot.wait(1000)
     assert shell.get_value('д') == 20
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
 def test_read_stderr(ipyconsole, qtbot):
     """
@@ -462,12 +269,8 @@ def test_read_stderr(ipyconsole, qtbot):
     assert content == client._read_stderr()
 
 
-@pytest.mark.slow
-@flaky(max_runs=10)
-@pytest.mark.no_xvfb
-@pytest.mark.skipif(os.environ.get('CI', None) is not None and os.name == 'nt',
-                    reason="It times out on AppVeyor.")
-@pytest.mark.timeout(timeout=20, method='thread')
+@flaky(max_runs=3)
+@pytest.mark.skipif(True, reason="It times out too much (to check later)")
 def test_values_dbg(ipyconsole, qtbot):
     """
     Test that getting, setting, copying and removing values is working while
@@ -484,7 +287,7 @@ def test_values_dbg(ipyconsole, qtbot):
     with qtbot.waitSignal(shell.executed):
         shell.execute('1/0')
     shell.execute('%debug')
-    qtbot.wait(1000)
+    qtbot.wait(500)
 
     # Get value
     qtbot.keyClicks(control, '!aa = 10')
@@ -493,7 +296,7 @@ def test_values_dbg(ipyconsole, qtbot):
     assert shell.get_value('aa') == 10
 
     # Set value
-    shell.set_value('aa', [cloudpickle.dumps(20, protocol=2)])
+    shell.set_value('aa', serialize_object(20))
     qtbot.wait(1000)
     assert shell.get_value('aa') == 20
 
@@ -511,8 +314,7 @@ def test_values_dbg(ipyconsole, qtbot):
     assert "*** NameError: name 'aa' is not defined" in control.toPlainText()
 
 
-@pytest.mark.slow
-@flaky(max_runs=10)
+@flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt', reason="It doesn't work on Windows")
 def test_plot_magic_dbg(ipyconsole, qtbot):
     """Test our plot magic while debugging"""
@@ -531,7 +333,7 @@ def test_plot_magic_dbg(ipyconsole, qtbot):
     with qtbot.waitSignal(shell.executed):
         shell.execute('1/0')
     shell.execute('%debug')
-    qtbot.wait(1000)
+    qtbot.wait(500)
 
     # Test reset magic
     qtbot.keyClicks(control, '%plot plt.plot(range(10))')
@@ -542,7 +344,6 @@ def test_plot_magic_dbg(ipyconsole, qtbot):
     assert shell._control.toHtml().count('img src') == 1
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
 def test_run_doctest(ipyconsole, qtbot):
@@ -579,9 +380,8 @@ def test_run_doctest(ipyconsole, qtbot):
     assert "TestResults(failed=0, attempted=2)" in shell._control.toPlainText()
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt' or (PY2 and PYQT5) or PYQT4,
+@pytest.mark.skipif(os.name == 'nt' or (PY2 and PYQT5),
                     reason="It times out frequently")
 def test_mpl_backend_change(ipyconsole, qtbot):
     """
@@ -611,9 +411,8 @@ def test_mpl_backend_change(ipyconsole, qtbot):
     assert shell._control.toHtml().count('img src') == 1
 
 
-@flaky(max_runs=10)
-@pytest.mark.skipif(os.environ.get('CI', None) is not None or PYQT5,
-                    reason="It fails frequently in PyQt5 and our CIs")
+@flaky(max_runs=3)
+@pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
 def test_ctrl_c_dbg(ipyconsole, qtbot):
     """
     Test that Ctrl+C works while debugging
@@ -630,16 +429,15 @@ def test_ctrl_c_dbg(ipyconsole, qtbot):
         shell.execute('1/0')
 
     shell.execute('%debug')
-    qtbot.wait(1000)
+    qtbot.wait(500)
 
     # Test Ctrl+C
     qtbot.keyClick(control, Qt.Key_C, modifier=Qt.ControlModifier)
-    qtbot.wait(2000)
+    qtbot.wait(500)
     assert 'For copying text while debugging, use Ctrl+Shift+C' in control.toPlainText()
 
 
-@pytest.mark.slow
-@flaky(max_runs=10)
+@flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt', reason="It doesn't work on Windows")
 def test_clear_and_reset_magics_dbg(ipyconsole, qtbot):
     """
@@ -657,11 +455,11 @@ def test_clear_and_reset_magics_dbg(ipyconsole, qtbot):
         shell.execute('1/0')
 
     shell.execute('%debug')
-    qtbot.wait(1000)
+    qtbot.wait(500)
 
     # Test clear magic
     shell.clear_console()
-    qtbot.wait(1000)
+    qtbot.wait(500)
     assert '\nipdb> ' == control.toPlainText()
 
     # Test reset magic
@@ -670,7 +468,7 @@ def test_clear_and_reset_magics_dbg(ipyconsole, qtbot):
     qtbot.wait(500)
     assert shell.get_value('bb') == 10
 
-    shell.reset_namespace(warning=False, silent=True)
+    shell.reset_namespace(force=True)
     qtbot.wait(1000)
 
     qtbot.keyClicks(control, '!bb')
@@ -680,7 +478,6 @@ def test_clear_and_reset_magics_dbg(ipyconsole, qtbot):
     assert "*** NameError: name 'bb' is not defined" in control.toPlainText()
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
 def test_restart_kernel(ipyconsole, qtbot):
@@ -691,7 +488,7 @@ def test_restart_kernel(ipyconsole, qtbot):
     client = ipyconsole.get_current_client()
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
-    # Do an assignment to verify that it's not there after restarting
+    # Do an assigment to verify that it's not there after restarting
     with qtbot.waitSignal(shell.executed):
         shell.execute('a = 10')
 
@@ -704,7 +501,6 @@ def test_restart_kernel(ipyconsole, qtbot):
     assert not shell.is_defined('a')
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
 def test_load_kernel_file_from_id(ipyconsole, qtbot):
@@ -727,10 +523,8 @@ def test_load_kernel_file_from_id(ipyconsole, qtbot):
     assert new_client.id_ == dict(int_id='1', str_id='B')
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt' or (PY3 and PYQT4),
-                    reason="It segfaults frequently")
+@pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
 def test_load_kernel_file_from_location(ipyconsole, qtbot):
     """
     Test that a new client is created using a connection file
@@ -753,10 +547,8 @@ def test_load_kernel_file_from_location(ipyconsole, qtbot):
     assert len(ipyconsole.get_clients()) == 2
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt' or PYQT4,
-                    reason="It segfaults frequently")
+@pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
 def test_load_kernel_file(ipyconsole, qtbot):
     """
     Test that a new client is created using the connection file
@@ -781,7 +573,6 @@ def test_load_kernel_file(ipyconsole, qtbot):
     assert shell.get_value('a') == new_shell.get_value('a')
 
 
-@pytest.mark.slow
 @flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
 def test_sys_argv_clear(ipyconsole, qtbot):
