@@ -13,7 +13,8 @@ import os.path as osp
 from qtpy.QtCore import Signal, QEvent, QObject, QRegExp, QSize, Qt
 from qtpy.QtGui import (QIcon, QRegExpValidator, QTextCursor)
 from qtpy.QtWidgets import (QDialog, QHBoxLayout, QLabel, QLineEdit,
-                            QListWidget, QListWidgetItem, QVBoxLayout)
+                            QListWidget, QListWidgetItem, QVBoxLayout,
+                            QMainWindow)
 
 # Local imports
 from spyder.config.base import _
@@ -186,9 +187,7 @@ def shorten_paths(path_list, is_unsaved):
 
 
 class KeyPressFilter(QObject):
-    """
-    Use with `installEventFilter` to get up/down arrow key press signal.
-    """
+    """Use with `installEventFilter` to get up/down arrow key press signal."""
     UP, DOWN = [-1, 1]  # Step constants
 
     sig_up_key_pressed = Signal()
@@ -204,26 +203,46 @@ class KeyPressFilter(QObject):
         return super(KeyPressFilter, self).eventFilter(src, e)
 
 
+class FilesFilterLine(QLineEdit):
+    """QLineEdit used to filter files by name."""
+
+    # User has not clicked outside this widget
+    clicked_outside = False
+
+    def focusOutEvent(self, event):
+        """
+        Detect when the focus goes out of this widget.
+
+        This is used to make the file switcher leave focus on the
+        last selected file by the user.
+        """
+        self.clicked_outside = True
+        return super(QLineEdit, self).focusOutEvent(event)
+
+
 class FileSwitcher(QDialog):
     """A Sublime-like file switcher."""
-    sig_goto_file = Signal(int)
-    sig_close_file = Signal(int)
+    sig_goto_file = Signal(int, object)
 
     # Constants that define the mode in which the list widget is working
     # FILE_MODE is for a list of files, SYMBOL_MODE if for a list of symbols
     # in a given file when using the '@' symbol.
     FILE_MODE, SYMBOL_MODE = [1, 2]
+    MAX_WIDTH = 600
 
-    def __init__(self, parent, tabs, data):
+    def __init__(self, parent, plugin, tabs, data, icon):
         QDialog.__init__(self, parent)
 
         # Variables
-        self.tabs = tabs                  # Editor stack tabs
-        self.data = data                  # Editor data
+        self.plugins_tabs = []
+        self.plugins_data = []
+        self.plugins_instances = []
+        self.add_plugin(plugin, tabs, data, icon)
+        self.plugin = None                # Last plugin with focus
         self.mode = self.FILE_MODE        # By default start in this mode
         self.initial_cursors = None       # {fullpath: QCursor}
         self.initial_path = None          # Fullpath of initial active editor
-        self.initial_editor = None        # Initial active editor
+        self.initial_widget = None        # Initial active editor
         self.line_number = None           # Selected line number in filer
         self.is_visible = False           # Is the switcher visible?
 
@@ -240,7 +259,7 @@ class FileSwitcher(QDialog):
                         "([A-Za-z0-9_]{0,100}:{0,1}[0-9]{0,100})")
 
         # Widgets
-        self.edit = QLineEdit(self)
+        self.edit = FilesFilterLine(self)
         self.help = HelperToolButton()
         self.list = QListWidget(self)
         self.filter = KeyPressFilter()
@@ -272,44 +291,62 @@ class FileSwitcher(QDialog):
         self.list.itemSelectionChanged.connect(self.item_selection_changed)
         self.list.clicked.connect(self.edit.setFocus)
 
-        # Setup
-        self.save_initial_state()
-        self.set_dialog_position()
-        self.setup()
-
     # --- Properties
     @property
-    def editors(self):
-        return [self.tabs.widget(index) for index in range(self.tabs.count())]
+    def widgets(self):
+        widgets = []
+        for tabs, plugin in self.plugins_tabs:
+            widgets += [(tabs.widget(index), plugin) for
+                        index in range(tabs.count())]
+        return widgets
 
     @property
     def line_count(self):
-        return [editor.get_line_count() for editor in self.editors]
+        return [widget.get_line_count() for widget in self.widgets]
 
     @property
     def save_status(self):
-        return [getattr(td, 'newly_created', False) for td in self.data]
+        save_status = []
+        for da, icon in self.plugins_data:
+            save_status += [getattr(td, 'newly_created', False) for td in da]
+        return save_status
 
     @property
     def paths(self):
-        return [getattr(td, 'filename', None) for td in self.data]
+        paths = []
+        for da, icon in self.plugins_data:
+            paths += [getattr(td, 'filename', None) for td in da]
+        return paths
 
     @property
     def filenames(self):
-        return [os.path.basename(getattr(td, 'filename',
-                                         None)) for td in self.data]
+        filenames = []
+        for da, icon in self.plugins_data:
+            filenames += [os.path.basename(getattr(td,
+                                                   'filename',
+                                                   None)) for td in da]
+        return filenames
+
+    @property
+    def icons(self):
+        icons = []
+        for da, icon in self.plugins_data:
+            icons += [icon for td in da]
+        return icons
 
     @property
     def current_path(self):
-        return self.paths_by_editor[self.get_editor()]
+        return self.paths_by_widget[self.get_widget()]
 
     @property
-    def paths_by_editor(self):
-        return dict(zip(self.editors, self.paths))
+    def paths_by_widget(self):
+        widgets = [w[0] for w in self.widgets]
+        return dict(zip(widgets, self.paths))
 
     @property
-    def editors_by_path(self):
-        return dict(zip(self.paths, self.editors))
+    def widgets_by_path(self):
+        widgets = [w[0] for w in self.widgets]
+        return dict(zip(self.paths, widgets))
 
     @property
     def filter_text(self):
@@ -320,15 +357,20 @@ class FileSwitcher(QDialog):
         self.edit.setText(_str)
 
     def save_initial_state(self):
-        """Saves initial cursors and initial active editor."""
+        """Save initial cursors and initial active widget."""
         paths = self.paths
-        self.initial_editor = self.get_editor()
+        self.initial_widget = self.get_widget()
         self.initial_cursors = {}
 
-        for i, editor in enumerate(self.editors):
-            if editor is self.initial_editor:
+        for i, editor in enumerate(self.widgets):
+            if editor is self.initial_widget:
                 self.initial_path = paths[i]
-            self.initial_cursors[paths[i]] = editor.textCursor()
+            # This try is needed to make the fileswitcher work with 
+            # plugins that does not have a textCursor.
+            try:
+                self.initial_cursors[paths[i]] = editor.textCursor()
+            except AttributeError:
+                pass
 
     def accept(self):
         self.is_visible = False
@@ -339,40 +381,44 @@ class FileSwitcher(QDialog):
         """Restores initial cursors and initial active editor."""
         self.list.clear()
         self.is_visible = False
-        editors = self.editors_by_path
+        widgets = self.widgets_by_path
 
-        for path in self.initial_cursors:
-            cursor = self.initial_cursors[path]
-            if path in editors:
-                self.set_editor_cursor(editors[path], cursor)
+        if not self.edit.clicked_outside:
+            for path in self.initial_cursors:
+                cursor = self.initial_cursors[path]
+                if path in widgets:
+                    self.set_editor_cursor(widgets[path], cursor)
 
-        if self.initial_editor in self.paths_by_editor:
-            index = self.paths.index(self.initial_path)
-            self.sig_goto_file.emit(index)
+            if self.initial_widget in self.paths_by_widget:
+                index = self.paths.index(self.initial_path)
+                self.sig_goto_file.emit(index)
 
     def set_dialog_position(self):
-        """Positions the file switcher dialog in the center of the editor."""
+        """Positions the file switcher dialog."""
         parent = self.parent()
         geo = parent.geometry()
         width = self.list.width()  # This has been set in setup
-
         left = parent.geometry().width()/2 - width/2
-        top = 0
+        # Note: the +1 pixel on the top makes it look better
+        if isinstance(parent, QMainWindow):
+            top = (parent.toolbars_menu.geometry().height() +
+                   parent.menuBar().geometry().height() + 1)
+        else:
+            top = self.plugins_tabs[0][0].tabBar().geometry().height() + 1
+
         while parent:
             geo = parent.geometry()
             top += geo.top()
             left += geo.left()
             parent = parent.parent()
 
-        # Note: the +1 pixel on the top makes it look better
-        self.move(left, top + self.tabs.tabBar().geometry().height() + 1)
+        self.move(left, top)
 
-    def fix_size(self, content, extra=50):
+    def get_item_size(self, content):
         """
-        Adjusts the width and height of the file switcher,
-        based on its content.
+        Get the max size (width and height) for the elements of a list of
+        strings as a QLabel.
         """
-        # Update size of dialog based on longest shortened path
         strings = []
         if content:
             for rich_text in content:
@@ -381,20 +427,34 @@ class FileSwitcher(QDialog):
                 strings.append(label.text())
                 fm = label.fontMetrics()
 
-            # Max width
-            max_width = max([fm.width(s) * 1.3 for s in strings])
-            self.list.setMinimumWidth(max_width + extra)
+            return (max([fm.width(s) * 1.3 for s in strings]), fm.height())
 
-            # Max height
-            if len(strings) < 8:
-                max_entries = len(strings)
+    def fix_size(self, content):
+        """
+        Adjusts the width and height of the file switcher
+        based on the relative size of the parent and content.
+        """
+        # Update size of dialog based on relative size of the parent
+        if content:
+            width, height = self.get_item_size(content)
+
+            # Width
+            parent = self.parent()
+            relative_width = parent.geometry().width() * 0.65
+            if relative_width > self.MAX_WIDTH:
+                relative_width = self.MAX_WIDTH
+            self.list.setMinimumWidth(relative_width)
+
+            # Height
+            if len(content) < 8:
+                max_entries = len(content)
             else:
                 max_entries = 8
-            max_height = fm.height() * max_entries * 2.5
+            max_height = height * max_entries * 2.5
             self.list.setMinimumHeight(max_height)
 
-            # Set position according to size
-            self.set_dialog_position()
+            # Resize
+            self.list.resize(relative_width, self.list.height())
 
     # --- Helper methods: List widget
     def count(self):
@@ -420,24 +480,59 @@ class FileSwitcher(QDialog):
 
     def previous_row(self):
         """Select previous row in list widget."""
-        self.select_row(-1)
+        if self.mode == self.SYMBOL_MODE:
+            self.select_row(-1)
+            return
+        prev_row = self.current_row() - 1
+        if prev_row >= 0:
+            title = self.list.item(prev_row).text()
+        else:
+            title = ''
+        if prev_row == 0 and '</b></big><br>' in title:
+            self.list.scrollToTop()
+        elif '</b></big><br>' in title:
+            # Select the next previous row, the one following is a title
+            self.select_row(-2)
+        else:
+            self.select_row(-1)
 
     def next_row(self):
         """Select next row in list widget."""
-        self.select_row(+1)
+        if self.mode == self.SYMBOL_MODE:
+            self.select_row(+1)
+            return
+        next_row = self.current_row() + 1
+        if next_row < self.count():
+            if '</b></big><br>' in self.list.item(next_row).text():
+                # Select the next next row, the one following is a title
+                self.select_row(+2)
+            else:
+                self.select_row(+1)
 
-    # --- Helper methods: Editor
-    def get_editor(self, index=None, path=None):
-        """Get editor by index or path.
+    def get_stack_index(self, stack_index, plugin_index):
+        """Get the real index of the selected item."""
+        other_plugins_count = sum([other_tabs[0].count() \
+                                   for other_tabs in \
+                                   self.plugins_tabs[:plugin_index]])
+        real_index = stack_index - other_plugins_count
+
+        return real_index
+
+    # --- Helper methods: Widget
+    def get_widget(self, index=None, path=None, tabs=None):
+        """Get widget by index.
         
-        If no path or index specified the current active editor is returned
+        If no tabs and index specified the current active widget is returned.
         """
-        if index:
-            return self.tabs.widget(index)
-        elif path:
-            return self.tabs.widget(index)
+        if index and tabs:
+            return tabs.widget(index)
+        elif path and tabs:
+            return tabs.widget(index)
+        elif self.plugin:
+            index = self.plugins_instances.index(self.plugin)
+            return self.plugins_tabs[index][0].currentWidget()
         else:
-            return self.parent().get_current_editor()
+            return self.plugins_tabs[0][0].currentWidget()
 
     def set_editor_cursor(self, editor, cursor):
         """Set the cursor of an editor."""
@@ -456,14 +551,17 @@ class FileSwitcher(QDialog):
         """Go to specified line number in current active editor."""
         if line_number:
             line_number = int(line_number)
-            editor = self.get_editor()
-            editor.go_to_line(min(line_number, editor.get_line_count()))
+            editor = self.get_widget()
+            try:
+                editor.go_to_line(min(line_number, editor.get_line_count()))
+            except AttributeError:
+                pass
 
     # --- Helper methods: Outline explorer
     def get_symbol_list(self):
         """Get the list of symbols present in the file."""
         try:
-            oedata = self.get_editor().get_outlineexplorer_data()
+            oedata = self.get_widget().get_outlineexplorer_data()
         except AttributeError:
             oedata = {}
         return oedata
@@ -473,11 +571,26 @@ class FileSwitcher(QDialog):
         """List widget item selection change handler."""
         row = self.current_row()
         if self.count() and row >= 0:
+            if '</b></big><br>' in self.list.currentItem().text() and row == 0:
+                self.next_row()
             if self.mode == self.FILE_MODE:
                 try:
                     stack_index = self.paths.index(self.filtered_path[row])
-                    self.sig_goto_file.emit(stack_index)
+                    self.plugin = self.widgets[stack_index][1]
+                    plugin_index = self.plugins_instances.index(self.plugin)
+                    # Count the real index in the tabWidget of the
+                    # current plugin
+                    real_index = self.get_stack_index(stack_index,
+                                                      plugin_index)
+                    self.sig_goto_file.emit(real_index,
+                                            self.plugin.get_current_tab_manager())
                     self.goto_line(self.line_number)
+                    try:
+                        self.plugin.switch_to_plugin()
+                        self.raise_()
+                    except AttributeError:
+                        # The widget using the fileswitcher is not a plugin
+                        pass
                     self.edit.setFocus()
                 except ValueError:
                     pass
@@ -489,6 +602,7 @@ class FileSwitcher(QDialog):
         """Setup list widget content for file list display."""
         short_paths = shorten_paths(self.paths, self.save_status)
         paths = self.paths
+        icons = self.icons
         results = []
         trying_for_line_number = ':' in filter_text
 
@@ -502,6 +616,10 @@ class FileSwitcher(QDialog):
         scores = get_search_scores(filter_text, self.filenames,
                                    template="<b>{0}</b>")
 
+        # Get max width to determine if shortpaths should be used
+        max_width = self.get_item_size(paths)[0]
+        self.fix_size(paths)
+
         # Build the text that will appear on the list widget
         for index, score in enumerate(scores):
             text, rich_text, score_value = score
@@ -510,16 +628,37 @@ class FileSwitcher(QDialog):
                 if trying_for_line_number:
                     text_item += " [{0:} {1:}]".format(self.line_count[index],
                                                        _("lines"))
-                text_item += u"<br><i>{0:}</i>".format(short_paths[index])
+                if max_width > self.list.width():
+                    text_item += u"<br><i>{0:}</i>".format(short_paths[index])
+                else:
+                    text_item += u"<br><i>{0:}</i>".format(paths[index])
                 results.append((score_value, index, text_item))
 
         # Sort the obtained scores and populate the list widget
         self.filtered_path = []
+        plugin = None
         for result in sorted(results):
             index = result[1]
-            text = result[-1]
             path = paths[index]
-            item = QListWidgetItem(ima.icon('FileIcon'), text)
+            icon = icons[index]
+            text = ''
+            try:
+                title = self.widgets[index][1].get_plugin_title().split(' - ')
+                if plugin != title[0]:
+                    plugin = title[0]
+                    text += '<br><big><b>' + plugin + '</b></big><br>'
+                    item = QListWidgetItem(text)
+                    item.setToolTip(path)
+                    item.setSizeHint(QSize(0, 25))
+                    item.setFlags(Qt.ItemIsEditable)
+                    self.list.addItem(item)
+                    self.filtered_path.append(path)
+            except:
+                # The widget using the fileswitcher is not a plugin
+                pass
+            text = ''
+            text += result[-1]
+            item = QListWidgetItem(icon, text)
             item.setToolTip(path)
             item.setSizeHint(QSize(0, 25))
             self.list.addItem(item)
@@ -533,7 +672,6 @@ class FileSwitcher(QDialog):
             self.set_current_row(self.filtered_path.index(current_path))
         elif self.filtered_path:
             self.set_current_row(0)
-        self.fix_size(short_paths, extra=200)
 
         # If a line number is searched look for it
         self.line_number = line_number
@@ -548,6 +686,13 @@ class FileSwitcher(QDialog):
         oedata = self.get_symbol_list()
         icons = get_python_symbol_icons(oedata)
 
+        # The list of paths here is needed in order to have the same
+        # point of measurement for the list widget size as in the file list
+        # See issue 4648
+        paths = self.paths
+        # Update list size
+        self.fix_size(paths)
+
         symbol_list = process_python_symbol_data(oedata)
         line_fold_token = [(item[0], item[2], item[3]) for item in symbol_list]
         choices = [item[1] for item in symbol_list]
@@ -589,12 +734,9 @@ class FileSwitcher(QDialog):
         # That's why this line is commented!
         # self.set_current_row(0)
 
-        # Update list size
-        self.fix_size(lines, extra=125)
-
     def setup(self):
         """Setup list widget content."""
-        if not self.tabs.count():
+        if len(self.plugins_tabs) == 0:
             self.close()
             return
 
@@ -611,3 +753,12 @@ class FileSwitcher(QDialog):
         else:
             self.mode = self.FILE_MODE
             self.setup_file_list(filter_text, current_path)
+
+        # Set position according to size
+        self.set_dialog_position()
+
+    def add_plugin(self, plugin, tabs, data, icon):
+        """Add a plugin to display its files."""
+        self.plugins_tabs.append((tabs, plugin))
+        self.plugins_data.append((data, icon))
+        self.plugins_instances.append(plugin)
