@@ -21,38 +21,46 @@ from qtpy.QtWidgets import (QApplication, QHBoxLayout, QInputDialog, QMenu,
                             QMessageBox, QToolButton, QVBoxLayout, QWidget)
 
 # Third party imports (others)
-import cloudpickle
-from spyder_kernels.utils.iofuncs import iofunctions
-from spyder_kernels.utils.misc import fix_reference_name
-from spyder_kernels.utils.nsview import get_supported_types, REMOTE_SETTINGS
+try:
+    import ipykernel.pickleutil
+    from ipykernel.serialize import serialize_object
+except ImportError:
+    serialize_object = None
 
 # Local imports
-from spyder.config.base import _, PICKLE_PROTOCOL
-from spyder.config.main import CONF
-from spyder.py3compat import is_text_string, to_text_string
+from spyder.config.base import _, get_supported_types
+from spyder.py3compat import is_text_string, getcwd, to_text_string
 from spyder.utils import encoding
 from spyder.utils import icon_manager as ima
-from spyder.utils.misc import getcwd_or_home
+from spyder.utils.iofuncs import iofunctions
+from spyder.utils.misc import fix_reference_name
 from spyder.utils.programs import is_module_installed
 from spyder.utils.qthelpers import (add_actions, create_action,
-                                    create_toolbutton, create_plugin_layout)
+                                    create_toolbutton)
 from spyder.widgets.variableexplorer.collectionseditor import (
     RemoteCollectionsEditorTableView)
 from spyder.widgets.variableexplorer.importwizard import ImportWizard
+from spyder.widgets.variableexplorer.utils import REMOTE_SETTINGS
 
 
 SUPPORTED_TYPES = get_supported_types()
 
-# Maximum length of a serialized variable to be set in the kernel
-MAX_SERIALIZED_LENGHT = 1e6
+# XXX --- Disable canning for Numpy arrays for now ---
+# This allows getting values between a Python 3 frontend
+# and a Python 2 kernel, and viceversa, for several types of
+# arrays.
+# See this link for interesting ideas on how to solve this
+# in the future:
+# http://stackoverflow.com/q/30698004/438386
+if serialize_object is not None:
+    ipykernel.pickleutil.can_map.pop('numpy.ndarray')
 
 
 class NamespaceBrowser(QWidget):
     """Namespace browser (global variables explorer widget)"""
     sig_option_changed = Signal(str, object)
     sig_collapse = Signal()
-    sig_free_memory = Signal()
-
+    
     def __init__(self, parent):
         QWidget.__init__(self, parent)
         
@@ -138,9 +146,9 @@ class NamespaceBrowser(QWidget):
 
         self.editor.sig_option_changed.connect(self.sig_option_changed.emit)
         self.editor.sig_files_dropped.connect(self.import_data)
-        self.editor.sig_free_memory.connect(self.sig_free_memory.emit)
 
         # Setup layout
+        layout = QVBoxLayout()
         blayout = QHBoxLayout()
         toolbar = self.setup_toolbar(exclude_private, exclude_uppercase,
                                      exclude_capitalized, exclude_unsupported)
@@ -163,9 +171,10 @@ class NamespaceBrowser(QWidget):
 
         blayout.addStretch()
         blayout.addWidget(options_button)
-
-        layout = create_plugin_layout(blayout, self.editor)
+        layout.addLayout(blayout)
+        layout.addWidget(self.editor)
         self.setLayout(layout)
+        layout.setContentsMargins(0, 0, 0, 0)
 
         self.sig_option_changed.connect(self.option_changed)
         
@@ -192,12 +201,8 @@ class NamespaceBrowser(QWidget):
                                            text=_("Save data as..."),
                                            icon=ima.icon('filesaveas'),
                                            triggered=self.save_data)
-        reset_namespace_button = create_toolbutton(
-                self, text=_("Remove all variables"),
-                icon=ima.icon('editdelete'), triggered=self.reset_namespace)
 
-        toolbar += [load_button, self.save_button, save_as_button,
-                    reset_namespace_button]
+        toolbar += [load_button, self.save_button, save_as_button]
         
         self.exclude_private_action = create_action(self,
                 _("Exclude private references"),
@@ -293,23 +298,8 @@ class NamespaceBrowser(QWidget):
 
     def set_value(self, name, value):
         """Set value for a variable."""
-        try:
-            # We need to enclose values in a list to be able to send
-            # them to the kernel in Python 2
-            svalue = [cloudpickle.dumps(value, protocol=PICKLE_PROTOCOL)]
-
-            # Needed to prevent memory leaks. See issue 7158
-            if len(svalue) < MAX_SERIALIZED_LENGHT:
-                self.shellwidget.set_value(name, svalue)
-            else:
-                QMessageBox.warning(self, _("Warning"),
-                                    _("The object you are trying to modify is "
-                                      "too big to be sent back to the kernel. "
-                                      "Therefore, your modifications won't "
-                                      "take place."))
-        except TypeError as e:
-            QMessageBox.critical(self, _("Error"),
-                                 "TypeError: %s" % to_text_string(e))
+        value = serialize_object(value)
+        self.shellwidget.set_value(name, value)
         self.refresh_table()
         
     def remove_values(self, names):
@@ -397,7 +387,7 @@ class NamespaceBrowser(QWidget):
         title = _("Import data")
         if filenames is None:
             if self.filename is None:
-                basedir = getcwd_or_home()
+                basedir = getcwd()
             else:
                 basedir = osp.dirname(self.filename)
             filenames, _selfilter = getopenfilenames(self, title, basedir,
@@ -460,20 +450,14 @@ class NamespaceBrowser(QWidget):
                                        "<br><br>Error message:<br>%s"
                                        ) % (self.filename, error_message))
             self.refresh_table()
-
-    @Slot()
-    def reset_namespace(self):
-        warning = CONF.get('ipython_console', 'show_reset_namespace_warning')
-        self.shellwidget.reset_namespace(warning=warning, silent=True,
-                                         message=True)
-
+            
     @Slot()
     def save_data(self, filename=None):
         """Save data"""
         if filename is None:
             filename = self.filename
             if filename is None:
-                filename = getcwd_or_home()
+                filename = getcwd()
             filename, _selfilter = getsavefilename(self, _("Save data"),
                                                    filename,
                                                    iofunctions.save_filters)
