@@ -5,19 +5,18 @@
 # (see spyder/__init__.py for details)
 
 """
-Utilities
+Utilities for the Collections editor widget and dialog
 """
 
 from __future__ import print_function
 
-from itertools import islice
 import re
 
 # Local imports
 from spyder.config.base import get_supported_types
 from spyder.py3compat import (NUMERIC_TYPES, TEXT_TYPES, to_text_string,
-                              is_text_string, is_binary_string, PY2,
-                              to_binary_string, iteritems)
+                              is_text_string, is_binary_string, reprlib,
+                              PY2, to_binary_string)
 from spyder.utils import programs
 from spyder import dependencies
 from spyder.config.base import _
@@ -153,6 +152,17 @@ def get_object_attrs(obj):
     return attrs
 
 
+# =============================================================================
+# Set limits for the amount of elements in the repr of collections (lists,
+# dicts, tuples and sets) and Numpy arrays
+# =============================================================================
+CollectionsRepr = reprlib.Repr()
+CollectionsRepr.maxlist = 10
+CollectionsRepr.maxdict = 10
+CollectionsRepr.maxtuple = 10
+CollectionsRepr.maxset = 10
+
+
 #==============================================================================
 # Date and datetime objects support
 #==============================================================================
@@ -246,71 +256,10 @@ def unsorted_unique(lista):
 #==============================================================================
 # Display <--> Value
 #==============================================================================
-def default_display(value, with_module=True):
-    """Default display for unknown objects."""
-    object_type = type(value)
-    try:
-        name = object_type.__name__
-        module = object_type.__module__
-        if with_module:
-            return name + ' object of ' + module + ' module'
-        else:
-            return name
-    except:
-        type_str = to_text_string(object_type)
-        return type_str[1:-1]
-
-
-def collections_display(value, level):
-    """Display for collections (i.e. list, tuple and dict)."""
-    is_dict = isinstance(value, dict)
-
-    # Get elements
-    if is_dict:
-        elements = iteritems(value)
-    else:
-        elements = value
-
-    # Truncate values
-    truncate = False
-    if level == 1 and len(value) > 10:
-        elements = islice(elements, 10) if is_dict else value[:10]
-        truncate = True
-    elif level == 2 and len(value) > 5:
-        elements = islice(elements, 5) if is_dict else value[:5]
-        truncate = True
-
-    # Get display of each element
-    if level <= 2:
-        if is_dict:
-            displays = [value_to_display(k, level=level) + ':' +
-                        value_to_display(v, level=level)
-                        for (k, v) in list(elements)]
-        else:
-            displays = [value_to_display(e, level=level)
-                        for e in elements]
-        if truncate:
-            displays.append('...')
-        display = ', '.join(displays)
-    else:
-        display = '...'
-
-    # Return display
-    if is_dict:
-        display = '{' + display + '}'
-    elif isinstance(value, list):
-        display = '[' + display + ']'
-    else:
-        display = '(' + display + ')'
-
-    return display
-
-
-def value_to_display(value, minmax=False, level=0):
+def value_to_display(value, minmax=False):
     """Convert value for display purpose"""
     # To save current Numpy threshold
     np_threshold = FakeObject
-
     try:
         numeric_numpy_types = (int64, int32, float64, float32,
                                complex128, complex64)
@@ -321,93 +270,68 @@ def value_to_display(value, minmax=False, level=0):
             # in our display
             set_printoptions(threshold=10)
         if isinstance(value, recarray):
-            if level == 0:
-                fields = value.names
-                display = 'Field names: ' + ', '.join(fields)
-            else:
-                display = 'Recarray'
+            fields = value.names
+            display = 'Field names: ' + ', '.join(fields)
         elif isinstance(value, MaskedArray):
             display = 'Masked array'
         elif isinstance(value, ndarray):
-            if level == 0:
-                if minmax:
-                    try:
-                        display = 'Min: %r\nMax: %r' % (value.min(), value.max())
-                    except (TypeError, ValueError):
-                        if value.dtype.type in numeric_numpy_types:
-                            display = repr(value)
-                        else:
-                            display = default_display(value)
-                elif value.dtype.type in numeric_numpy_types:
+            if minmax:
+                try:
+                    display = 'Min: %r\nMax: %r' % (value.min(), value.max())
+                except (TypeError, ValueError):
                     display = repr(value)
-                else:
-                    display = default_display(value)
             else:
-                display = 'Numpy array'
-        elif any([type(value) == t for t in [list, tuple, dict]]):
-            display = collections_display(value, level+1)
+                display = repr(value)
+        elif isinstance(value, (list, tuple, dict, set)):
+            display = CollectionsRepr.repr(value)
         elif isinstance(value, Image):
-            if level == 0:
-                display = '%s  Mode: %s' % (address(value), value.mode)
-            else:
-                display = 'Image'
+            display = '%s  Mode: %s' % (address(value), value.mode)
         elif isinstance(value, DataFrame):
-            if level == 0:
-                cols = value.columns
-                if PY2 and len(cols) > 0:
-                    # Get rid of possible BOM utf-8 data present at the
-                    # beginning of a file, which gets attached to the first
-                    # column header when headers are present in the first
-                    # row.
-                    # Fixes Issue 2514
-                    try:
-                        ini_col = to_text_string(cols[0], encoding='utf-8-sig')
-                    except:
-                        ini_col = to_text_string(cols[0])
-                    cols = [ini_col] + [to_text_string(c) for c in cols[1:]]
-                else:
-                    cols = [to_text_string(c) for c in cols]
-                display = 'Column names: ' + ', '.join(list(cols))
+            cols = value.columns
+            if PY2 and len(cols) > 0:
+                # Get rid of possible BOM utf-8 data present at the
+                # beginning of a file, which gets attached to the first
+                # column header when headers are present in the first
+                # row.
+                # Fixes Issue 2514
+                try:
+                    ini_col = to_text_string(cols[0], encoding='utf-8-sig')
+                except:
+                    ini_col = to_text_string(cols[0])
+                cols = [ini_col] + [to_text_string(c) for c in cols[1:]]
             else:
-                display = 'Dataframe'
+                cols = [to_text_string(c) for c in cols]
+            display = 'Column names: ' + ', '.join(list(cols))
         elif isinstance(value, NavigableString):
             # Fixes Issue 2448
             display = to_text_string(value)
-            if level > 0:
-                display = u"'" + display + u"'"
         elif isinstance(value, DatetimeIndex):
-            if level == 0:
-                display = value.summary()
-            else:
-                display = 'DatetimeIndex'
+            display = value.summary()
         elif is_binary_string(value):
             try:
                 display = to_text_string(value, 'utf8')
             except:
                 display = value
-            if level > 0:
-                display = (to_binary_string("'") + display +
-                           to_binary_string("'"))
         elif is_text_string(value):
             display = value
-            if level > 0:
-                display = u"'" + display + u"'"
-        elif (isinstance(value, NUMERIC_TYPES) or isinstance(value, bool) or
-              isinstance(value, datetime.date) or
-              isinstance(value, numeric_numpy_types)):
+        elif isinstance(value, NUMERIC_TYPES) or isinstance(value, bool) or \
+          isinstance(value, datetime.date) or \
+          isinstance(value, numeric_numpy_types):
             display = repr(value)
         else:
-            if level == 0:
-                display = default_display(value)
-            else:
-                display = default_display(value, with_module=False)
+            # Note: Don't trust on repr's. They can be inefficient and
+            # so freeze Spyder quite easily
+            # display = repr(value)
+            type_str = to_text_string(type(value))
+            display = type_str[1:-1]
     except:
-        display = default_display(value)
+        type_str = to_text_string(type(value))
+        display = type_str[1:-1]
 
-    # Truncate display at 70 chars to avoid freezing Spyder
+    # Truncate display at 80 chars to avoid freezing Spyder
     # because of large displays
-    if len(display) > 70:
-        display = display[:70].rstrip() + ' ...'
+    if len(display) > 80:
+        display = display[:80].rstrip() + ' ...'
 
     # Restore Numpy threshold
     if np_threshold is not FakeObject:
@@ -416,6 +340,7 @@ def value_to_display(value, minmax=False, level=0):
     return display
 
 
+
 def display_to_value(value, default_value, ignore_errors=True):
     """Convert back to value"""
     from qtpy.compat import from_qvariant
@@ -504,7 +429,7 @@ def get_human_readable_type(item):
 # Globals filter: filter namespace dictionaries (to be edited in
 # CollectionsEditor)
 #==============================================================================
-def is_supported(value, check_all=False, filters=None, iterate=False):
+def is_supported(value, check_all=False, filters=None, iterate=True):
     """Return True if the value is supported, False otherwise"""
     assert filters is not None
     if value is None:
